/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2024
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "ComponentPlugin.h"

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include <RobotAPI/interface/armem/mns/MemoryNameSystemInterface.h>
#include <RobotAPI/libraries/armem/client/MemoryNameSystem.h>
#include <RobotAPI/libraries/armem/client/plugins/Plugin.h>

namespace armarx::view_selection::client::plugins
{
    ComponentPlugin::ComponentPlugin(ManagedIceObject& parent, const std::string& pre) :
        armarx::ComponentPlugin(parent, pre)
    {
        if (armemPlugin_ == nullptr)
        {
            addPlugin(armemPlugin_);
        }
        addPluginDependency(armemPlugin_);

        const std::string componentName = parent.getName();

        viewSelection_.emplace(memoryNameSystem(), componentName);
    }

    // defined in cpp file in favor of forward declaration
    ComponentPlugin::~ComponentPlugin() = default;

    client::ViewSelection&
    ComponentPlugin::viewSelection()
    {
        ARMARX_CHECK_NOT_NULL(viewSelection_);
        return viewSelection_.value();
    }

    void
    ComponentPlugin::preOnConnectComponent()
    {
        viewSelection_->connect();
    }

    armarx::armem::client::MemoryNameSystem&
    ComponentPlugin::memoryNameSystem()
    {
        if (armemPlugin_ == nullptr)
        {
            addPlugin(armemPlugin_);
        }

        ARMARX_CHECK_NOT_NULL(armemPlugin_);
        return armemPlugin_->getMemoryNameSystemClient();
    }
} // namespace armarx::view_selection::client::plugins
