/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2024
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <ArmarXCore/core/ComponentPlugin.h>

#include <armarx/view_selection/client/ViewSelection.h>

// forward declaring privately used classes
namespace armarx::armem::client
{
    class MemoryNameSystem;

    namespace plugins
    {
        class Plugin;
    }
} // namespace armarx::armem::client

namespace armarx::view_selection::client::plugins
{
    class ComponentPlugin : public armarx::ComponentPlugin
    {
    public:
        ComponentPlugin(ManagedIceObject& parent, const std::string& pre);

        ~ComponentPlugin() override;

        client::ViewSelection& viewSelection();

    protected:
        void preOnConnectComponent() override;

    private:
        armarx::armem::client::MemoryNameSystem& memoryNameSystem();

        armarx::armem::client::plugins::Plugin* armemPlugin_ = nullptr;

        std::optional<client::ViewSelection> viewSelection_;
    };

} // namespace armarx::view_selection::client::plugins

// make class available in armarx::view_selection namespace
namespace armarx::view_selection
{
    using ComponentPlugin = client::plugins::ComponentPlugin;
}
