/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    view_selection::gaze_scheduler
 * @author     Johann Mantel ( j-mantel at gmx dot net )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <ArmarXCore/core/Component.h>
#include <ArmarXCore/core/time.h>
#include <ArmarXCore/util/CPPUtility/TripleBuffer.h>

#include <RobotAPI/libraries/RobotAPIComponentPlugins/ArVizComponentPlugin.h>
#include <RobotAPI/libraries/RobotAPIComponentPlugins/RobotUnitComponentPlugin.h>
#include <RobotAPI/libraries/armem/client/Reader.h>
#include <RobotAPI/libraries/armem/client/Writer.h>
#include <RobotAPI/libraries/armem/client/plugins/ListeningPluginUser.h>
#include <RobotAPI/libraries/armem/client/plugins/ReaderWriterPlugin.h>
#include <RobotAPI/libraries/armem_robot_state/client/common/VirtualRobotReader.h>
#include <RobotAPI/libraries/armem_robot_state/types.h>
#include <RobotAPI/libraries/core/FramedPose.h>

#include <armarx/control/client/ComponentPlugin.h>
#include <armarx/control/client/ControllerWrapper.h>
#include <armarx/view_selection/common/controller_types.h>
#include <armarx/view_selection/gaze_controller/controller_descriptions.h>
#include <armarx/view_selection/gaze_scheduler/ControllerHandlerInterface.h>
#include <armarx/view_selection/gaze_scheduler/Scheduler.h>
#include <armarx/view_selection/gaze_targets.h>
#include <armarx/view_selection/gaze_targets/GazeTarget.h>
#include <armarx/view_selection/interfaces/GazeSchedulerInterface.h>

namespace armarx::view_selection::gaze_scheduler
{

    class Component :
        virtual public armarx::Component,
        virtual public armarx::view_selection::gaze_scheduler::GazeSchedulerInterface,
        virtual public armarx::RobotUnitComponentPluginUser,
        virtual public armarx::armem::ListeningClientPluginUser,
        virtual public armarx::control::client::ComponentPluginUser,
        virtual public armarx::ArVizComponentPluginUser
    {
    public:
        Component()
        {
            addPlugin(virtualRobotReaderPlugin);
        }

        /// @see armarx::ManagedIceObject::getDefaultName()
        std::string getDefaultName() const override;

        /// Get the component's default name.
        static std::string GetDefaultName();

        // from GazeSchedulerInterface
        void resetPriorityQueue(const Ice::Current& c = ::Ice::Current()) override;


    protected:
        /// @see armarx::ManagedIceObject::onInitComponent()
        void onInitComponent() override;

        /// @see armarx::ManagedIceObject::onConnectComponent()
        void onConnectComponent() override;

        /// @see armarx::ManagedIceObject::onDisconnectComponent()
        void onDisconnectComponent() override;

        /// @see armarx::ManagedIceObject::onExitComponent()
        void onExitComponent() override;

        /// @see PropertyUser::createPropertyDefinitions()
        armarx::PropertyDefinitionsPtr createPropertyDefinitions() override;

        // MemoryListeningClient
        void onMemoryUpdated(const std::vector<armarx::armem::MemoryID>& updatedSnapshotIDs);

    private:
        static const std::string defaultName;

        struct Properties
        {
            common::ControllerType controllerType = common::ControllerType::GazeControllerGazeIK;

            std::string defaultTarget = "0,1500,1500";

            bool visuEnabled = false;
            int targetVizAlpha = 128;

            bool dummy = false;

            bool rtUnitDynamicallyLoadLibraries = false;
        };

        Properties properties;

        armem::client::plugins::ReaderWriterPlugin<armem::robot_state::VirtualRobotReader>*
            virtualRobotReaderPlugin = nullptr;


        std::optional<Scheduler> scheduler;
        std::unique_ptr<ControllerHandlerInterface> controllerHandler;
    };


} // namespace armarx::view_selection::gaze_scheduler
