/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    view_selection::ArmarXObjects::memory
 * @author     Johann Mantel ( j-mantel at gmx dot net )
 * @date       2022
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <cstdint>

#include <ArmarXCore/core/Component.h>

#include <RobotAPI/libraries/armem/server/plugins/ReadWritePluginUser.h>
#include <RobotAPI/libraries/armem/server/wm/memory_definitions.h>

#include <armarx/view_selection/components/memory/ComponentInterface.h>

namespace armarx::view_selection::components::memory
{

    class Component :
        virtual public armarx::Component,
        virtual public armarx::view_selection::components::memory::ComponentInterface,
        virtual public armarx::armem::server::ReadWritePluginUser
    {
    public:
        /// @see armarx::ManagedIceObject::getDefaultName()
        std::string getDefaultName() const override;

        /// Get the component's default name.
        static std::string GetDefaultName();

    protected:
        /// @see PropertyUser::createPropertyDefinitions()
        armarx::PropertyDefinitionsPtr createPropertyDefinitions() override;

        /// @see armarx::ManagedIceObject::onInitComponent()
        void onInitComponent() override;

        /// @see armarx::ManagedIceObject::onConnectComponent()
        void onConnectComponent() override;

        /// @see armarx::ManagedIceObject::onDisconnectComponent()
        void onDisconnectComponent() override;

        /// @see armarx::ManagedIceObject::onExitComponent()
        void onExitComponent() override;

    private:
        // Private methods go here.

    private:
        static const std::string defaultName;

        // Private member variables go here.

        /// Properties shown in the Scenario GUI.

        struct CoreSegmentProperties
        {
            std::int64_t maxHistorySize = 100;
        };

        struct Properties
        {
            struct CoreSegments
            {
                CoreSegmentProperties gazeTarget;
            } coreSeg;
        };

        Properties properties;
    };

} // namespace armarx::view_selection::components::memory
