#include "LookAt.h"

#include <Eigen/Core>
#include <Eigen/Geometry>

#include <armarx/view_selection/gaze_targets/GazeTarget.h>
#include <armarx/view_selection/skills/skill_ids.h>

namespace armarx::view_selection::skills
{

    LookAt::LookAt(const Context& services) : Base(DefaultSkillDescription()), srv_(services)
    {
    }

    ::armarx::skills::Skill::InitResult
    LookAt::init(const Base::SpecializedInitInput& in)
    {
        gazeTarget.emplace(); // populate optional

        fromAron(in.parameters.target, gazeTarget.value());

        return ::armarx::skills::Skill::InitResult{
            .status = ::armarx::skills::TerminatedSkillStatus::Succeeded};
    }

    ::armarx::skills::Skill::MainResult
    LookAt::main(const Base::SpecializedMainInput& in)
    {
        ARMARX_CHECK(gazeTarget.has_value());

        ARMARX_INFO << "Committing gaze target " << gazeTarget.value();
        srv_->viewSelectionClient.commitGazeTarget(gazeTarget.value());

        return ::armarx::skills::Skill::MainResult{
            .status = ::armarx::skills::TerminatedSkillStatus::Succeeded};
    }

    void
    LookAt::onStopRequested()
    {
        if (gazeTarget.has_value())
        {
            srv_->viewSelectionClient.commitGazeTarget(gazeTarget.value());
        }
    }

    ::armarx::skills::SkillDescription
    LookAt::DefaultSkillDescription()
    {
        ParamType defaultParameters;
        defaultParameters.target.duration = armarx::Duration::Seconds(0);
        defaultParameters.target.name = "";
        defaultParameters.target.keepInQueue = false;

        return armarx::skills::SkillDescription{
            .skillId = skill_ids::LookAt,
            .description = "",
            .rootProfileDefaults = defaultParameters.toAron(),
            .timeout = armarx::core::time::Duration::Seconds(10),
            .parametersType = Params::ToAronType(),
        };
    }


} // namespace armarx::view_selection::skills
