/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2023
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <ArmarXCore/core/ManagedIceObject.h>
#include <ArmarXCore/core/time/Duration.h>

#include <RobotAPI/libraries/armem_robot_state/client/common/RobotReader.h> // for hand
#include <RobotAPI/libraries/skills/provider/SimpleSkill.h>
#include <RobotAPI/libraries/skills/provider/SimpleSpecializedSkill.h>

#include <armarx/view_selection/client/ViewSelection.h>
#include <armarx/view_selection/skills/aron/LookAtHumanHand.aron.generated.h>
#include <armarx/view_selection/skills/constants.h>
#include <armarx/view_selection/target_provider/handover/RobotReceiver.h>

#include <VisionX/libraries/armem_human/client/HumanPoseReader.h>

namespace armarx::view_selection::skills
{
    class LookAtHumanHand :
        virtual public armarx::skills::SimpleSpecializedSkill<arondto::LookAtHumanHandParams>
    {
    public:
        using Params = arondto::LookAtHumanHandParams;
        using Base = ::armarx::skills::SimpleSpecializedSkill<Params>;

        struct Services
        {
            armem::client::MemoryNameSystem& memoryNameSystem;
            client::ViewSelection& viewSelectionClient;
            const armarx::armem::MemoryID humanPoseMemoryID;
            const VirtualRobot::RobotPtr& robot;
        };

        LookAtHumanHand(const Services& srv); // FIXME add params

    private:
        ::armarx::skills::Skill::InitResult init(const Base::SpecializedInitInput& in) override;

        ::armarx::skills::Skill::MainResult main(const Base::SpecializedMainInput& in) override;

        void update();
        std::optional<armem::human::HumanPose> getLatestPoseOfRequestedHuman();

        void onStopRequested() override;

    public:
        static armarx::skills::SkillDescription
        DefaultSkillDescription()
        {
            ParamType defaultParameters;
            // TODO(): set parameter defaults

            return armarx::skills::SkillDescription{
                .skillId = {.skillName = constants::skill_names::LookAtHumanHand},
                .description = "Look at the face of a human specified by their tracking ID.",
                .rootProfileDefaults = defaultParameters.toAron(),
                .timeout = armarx::core::time::Duration::Seconds(10),
                .parametersType = Params::ToAronType()};
        }

    protected:
    private:
        std::optional<Services> srv_;

        std::string humanTrackingId;
        armarx::armem::robot_state::RobotReader::Hand hand;
        std::unique_ptr<target_provider::handover::RobotReceiver> handoverTargetProvider;

        struct
        {
            double updatePeriodMs = 1000. / 30.;
        } properties;
    };

} // namespace armarx::view_selection::skills
