/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2023
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <VirtualRobot/VirtualRobot.h>

#include <RobotAPI/interface/core/RobotState.h>
#include <RobotAPI/interface/speech/SpeechInterface.h>
#include <RobotAPI/interface/units/KinematicUnitInterface.h>
#include <RobotAPI/libraries/ArmarXObjects/ObjectPoseClient.h>
#include <RobotAPI/libraries/skills/provider/SimpleSpecializedSkill.h>

#include <armarx/view_selection/skills/aron/LookForObjects.aron.generated.h>

namespace armarx::view_selection::skills
{
    class LookForObjectsWithKinematicUnit :
        virtual public armarx::skills::SimpleSpecializedSkill<arondto::LookForObjectsParams>
    {
    public:
        using Params = arondto::LookForObjectsParams;
        using Base = ::armarx::skills::SimpleSpecializedSkill<Params>;

        struct Services
        {
            KinematicUnitInterfacePrx kinematicUnit;

            objpose::ObjectPoseClient objectPoseClient;
            armarx::TextListenerInterfacePrx textToSpeech;
            armarx::RobotStateComponentInterfacePrx robotStateComponent;

            // armem::client::MemoryNameSystem& memoryNameSystem;
        };

        LookForObjectsWithKinematicUnit(const Services& srv);

    private:
        ::armarx::skills::Skill::InitResult init(const Base::SpecializedInitInput& in) override;
        ::armarx::skills::Skill::MainResult main(const Base::SpecializedMainInput& in) override;

        void onStopRequested() override;


    public:
        static armarx::skills::SkillDescription DefaultSkillDescription();

        std::optional<objpose::ObjectPose> findObject(const objpose::ObjectPoseSeq& objectPoses,
                                                      const armarx::ObjectID& objectID);
        void setNeckJointAngles(float goal_yaw, float goal_pitch, int waypoints_num);
        void speak(const std::string& text);


    protected:
    private:
        std::optional<Services> srv_;

        // will be initialized in "init()"
    };

} // namespace armarx::view_selection::skills
