#include "ResetGazeTargets.h"

#include <Eigen/Core>
#include <Eigen/Geometry>

#include <armarx/view_selection/gaze_targets/GazeTarget.h>
#include <armarx/view_selection/skills/constants.h>
#include <armarx/view_selection/skills/skill_ids.h>

namespace armarx::view_selection::skills
{

    ResetGazeTargets::ResetGazeTargets(const Context& services) :
        Base(DefaultSkillDescription()), srv_(services)
    {
        ARMARX_CHECK_NOT_NULL(srv_->viewSelectionClient);
    }

    // ::armarx::skills::Skill::InitResult
    // ResetGazeTargets::init(const Base::InitInput& in)
    // {
    //     return ::armarx::skills::Skill::InitResult{
    //         .status = ::armarx::skills::TerminatedSkillStatus::Succeeded};
    // }

    // ::armarx::skills::Skill::MainResult
    // ResetGazeTargets::main(const Base::MainInput& in)
    // {
    //     ARMARX_INFO << "Resetting all gaze targets";
    //     ARMARX_CHECK_NOT_NULL(srv_->viewSelectionClient);
    //     srv_->viewSelectionClient->resetPriorityQueue();

    //     return ::armarx::skills::Skill::MainResult{
    //         .status = ::armarx::skills::TerminatedSkillStatus::Succeeded};
    // }

    ::armarx::skills::Skill::InitResult
    ResetGazeTargets::init(const Base::SpecializedInitInput& in)
    {
        return ::armarx::skills::Skill::InitResult{
            .status = ::armarx::skills::TerminatedSkillStatus::Succeeded};
    }

    ::armarx::skills::Skill::MainResult
    ResetGazeTargets::main(const Base::SpecializedMainInput& in)
    {
        ARMARX_INFO << "Resetting all gaze targets";
        ARMARX_CHECK_NOT_NULL(srv_->viewSelectionClient);
        srv_->viewSelectionClient->resetPriorityQueue();

        return ::armarx::skills::Skill::MainResult{
            .status = ::armarx::skills::TerminatedSkillStatus::Succeeded};
    }

    void
    ResetGazeTargets::onStopRequested()
    {
    }

    ::armarx::skills::SkillDescription
    ResetGazeTargets::DefaultSkillDescription()
    {
        Params defaults;

        return armarx::skills::SkillDescription{
            .skillId = skill_ids::ResetGazeTargets,
            .description = "",
            .rootProfileDefaults = defaults.toAron(),
            .timeout = armarx::core::time::Duration::Seconds(10),
            .parametersType = Params::ToAronType(),
        };
    }


} // namespace armarx::view_selection::skills
