/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2023
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <ArmarXCore/core/time/Duration.h>

#include <RobotAPI/libraries/skills/provider/SimpleSkill.h>
#include <RobotAPI/libraries/skills/provider/SimpleSpecializedSkill.h>

#include <armarx/view_selection/client/ViewSelection.h>
#include <armarx/view_selection/interfaces/GazeSchedulerInterface.h>
#include <armarx/view_selection/skills/aron/ResetGazeTargets.aron.generated.h>

namespace armarx::view_selection::skills
{
    class ResetGazeTargets :
        virtual public armarx::skills::SimpleSpecializedSkill<arondto::ResetGazeTargetsParams>
    {
    public:
        using Params = arondto::ResetGazeTargetsParams;
        // using Result = arondto::LookForObjectsResult;
        // using Base = ::armarx::skills::SimpleSkill;
        using Base = ::armarx::skills::SimpleSpecializedSkill<Params>;

        struct Context
        {
            gaze_scheduler::GazeSchedulerInterfacePrx& viewSelectionClient;
        };

        ResetGazeTargets(const Context& services);

        static armarx::skills::SkillDescription DefaultSkillDescription();

    private:
        // ::armarx::skills::Skill::InitResult init(const Base::InitInput& in) override;

        // ::armarx::skills::Skill::MainResult main(const Base::MainInput& in) override;

        ::armarx::skills::Skill::InitResult init(const Base::SpecializedInitInput& in) override;
        ::armarx::skills::Skill::MainResult main(const Base::SpecializedMainInput& in) override;


        void onStopRequested() override;

    private:
        std::optional<Context> srv_;
    };

} // namespace armarx::view_selection::skills
