/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    view_selection::ViewSelectionGroup
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2022
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "CreateTaskLevelGazeTarget.h"

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>
#include <ArmarXCore/core/logging/Logging.h>

#include <RobotAPI/libraries/armem/client/MemoryNameSystem.h>

#include <armarx/view_selection/client/ViewSelection.h>

namespace armarx::view_selection::statecharts::view_selection_group
{
    // DO NOT EDIT NEXT LINE
    CreateTaskLevelGazeTarget::SubClassRegistry
        CreateTaskLevelGazeTarget::Registry(CreateTaskLevelGazeTarget::GetName(),
                                            &CreateTaskLevelGazeTarget::CreateInstance);

    void
    CreateTaskLevelGazeTarget::onEnter()
    {
        // put your user code for the enter-point here
        // execution time should be short (<100ms)
    }

    void
    CreateTaskLevelGazeTarget::run()
    {
        ARMARX_CHECK_GREATER_EQUAL(in.getpriority(), 0);
        ARMARX_CHECK_GREATER_EQUAL(in.getdurationMs(), 0);

        ARMARX_CHECK_NOT_NULL(in.gettarget());

        armem::client::MemoryNameSystem mns(getMemoryNameSystem());
        client::ViewSelection viewSelection(mns, CreateTaskLevelGazeTarget::GetName());

        ARMARX_INFO << "Connecting view selection";
        viewSelection.connect();

        gaze_targets::GazeTarget target;
        target.name = "look_at_target";
        target.position = *in.gettarget();
        target.priority =
            gaze_targets::TargetPriority(gaze_targets::AttentionType::TaskDriven, in.getpriority());
        target.duration = in.getdurationMs() > 0 ? Duration::MilliSecondsDouble(in.getdurationMs())
                                                 : Duration::MilliSeconds(-1);
        target.creationTimestamp = armarx::Clock::Now();

        ARMARX_IMPORTANT << "Committing gaze target " << target;
        viewSelection.commitGazeTarget(target);
    }

    //void CreateTaskLevelGazeTarget::onBreak()
    //{
    //    // put your user code for the breaking point here
    //    // execution time should be short (<100ms)
    //}

    void
    CreateTaskLevelGazeTarget::onExit()
    {
        // put your user code for the exit point here
        // execution time should be short (<100ms)
    }

    // DO NOT EDIT NEXT FUNCTION
    XMLStateFactoryBasePtr
    CreateTaskLevelGazeTarget::CreateInstance(XMLStateConstructorParams stateData)
    {
        return {new CreateTaskLevelGazeTarget(stateData)};
    }
} // namespace armarx::view_selection::statecharts::view_selection_group
