/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2022
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <VirtualRobot/VirtualRobot.h>

#include <armarx/view_selection/gaze_targets/GazeTarget.h>
#include <armarx/view_selection/target_provider/handover/types.h>

#include <VisionX/libraries/armem_human/types.h>

namespace armarx::view_selection::target_provider::handover
{

    class HandoverTargetProvider
    {
    public:
        HandoverTargetProvider(const std::string& humanTrackingId,
                               const VirtualRobot::RobotPtr& robot);
        virtual ~HandoverTargetProvider() = default;


        virtual void setPhase(Phase phase);

        virtual std::vector<gaze_targets::GazeTarget>
        updateTargets(const armem::human::HumanPose& human) = 0;

    protected:
        const std::string humanTrackingId_;
        const VirtualRobot::RobotPtr robot_;

        Phase phase_ = Phase::PreHandover;
    };
} // namespace armarx::view_selection::target_provider::handover
