/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2022
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <armarx/view_selection/gaze_targets/GazeTarget.h>
#include <armarx/view_selection/target_provider/handover/HandoverTargetProvider.h>
#include <armarx/view_selection/target_provider/handover/types.h>

namespace armarx::view_selection::target_provider::handover
{

    /**
    * @brief The robot is acting as the giver
    * 
    * The giver will first look at the receiver's face during reach. At a certain distance to the receiver, it will then look at the receiver's hand.
    * Once the object is exchanged, the giver will then look at the receivers face.
    */
    class RobotGiver : public HandoverTargetProvider
    {
    public:
        using HandoverTargetProvider::HandoverTargetProvider;

        struct Params
        {

            //! the distance below which the hand targets will be generated.
            float maxHandoverInitializationDistance = 3000;

            float minHandReachoutDistance = 300;
            float maxHandReachoutDistance = 1000;

            float handPriorityScaling = 1000;

            float headPriority = 100;

            float minHandoverDistance = 500;
        };

        std::vector<gaze_targets::GazeTarget>
        updateTargets(const armem::human::HumanPose& human) override;

    protected:
    private:
        std::vector<gaze_targets::GazeTarget>
        updateTargetsPreHandover(const armem::human::HumanPose& human) const;

        std::vector<gaze_targets::GazeTarget>
        updateTargetsAfterHandover(const armem::human::HumanPose& human) const;

        const Params params;

        Phase phase_;
    };
} // namespace armarx::view_selection::target_provider::handover
