/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2022
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <armarx/view_selection/target_provider/handover/HandoverTargetProvider.h>
#include <armarx/view_selection/target_provider/handover/types.h>

namespace armarx::view_selection::target_provider::handover
{

    /**
    * @brief The robot is acting as the receiver
    * 
     * The receiver will look at the giver's hand during reach and transfer phase. Once it received the object, it will look at the giver's face.
    * 
    */
    class RobotReceiver : public HandoverTargetProvider
    {
    public:
        using HandoverTargetProvider::HandoverTargetProvider;

        struct Params
        {

            //! the distance below which the hand targets will be generated.
            float maxHandoverInitializationDistance = 3000;

            float maxHandsInitializationDistance = 2500;

            float minHandReachoutDistance = 300;
            float maxHandReachoutDistance = 1000;

            float handPriorityScaling = 1000;

            float headPriority = 100;

            float minHandoverDistance = 500;
        };

        RobotReceiver(const std::string& humanTrackingId, const VirtualRobot::RobotPtr& robot);

        std::vector<gaze_targets::GazeTarget>
        updateTargets(const armem::human::HumanPose& human) override;

        // TODO: used to be private, but is now also needed by lookAtHumanFace/lookAtHumanHand skills
        /*private:*/
        std::vector<gaze_targets::GazeTarget>
        updateTargetsPreHandover(const armem::human::HumanPose& human) const;

        std::vector<gaze_targets::GazeTarget>
        updateTargetsAfterHandover(const armem::human::HumanPose& human) const;

        const Params params;
    };
} // namespace armarx::view_selection::target_provider::handover
