#include "util.h"

#include <optional>

#include <ArmarXCore/core/logging/Logging.h>

#include <VisionX/libraries/human/pose/model/k4a_bt_body_32.h>
#include <VisionX/libraries/human/pose/model/openpose_body_25.h>

namespace armarx::view_selection::target_provider::handover::util
{
    std::optional<std::string>
    headKeypoint(const std::string& poseModelId)
    {
        namespace k4a_bt_body_32 = human::pose::model::k4a_bt_body_32;
        namespace openpose_body_25 = human::pose::model::openpose_body_25;

        if (poseModelId == openpose_body_25::ModelId)
        {
            return openpose_body_25::JointNames.to_name(openpose_body_25::Joints::Nose);
        }

        if (poseModelId == k4a_bt_body_32::ModelId)
        {
            return k4a_bt_body_32::JointNames.to_name(k4a_bt_body_32::Joints::Head);
        }

        ARMARX_ERROR << "Unknown pose model ID '" << poseModelId << "'.";
        return std::nullopt;
    }

    std::optional<std::string>
    leftWristKeypoint(const std::string& poseModelId)
    {
        namespace k4a_bt_body_32 = human::pose::model::k4a_bt_body_32;
        namespace openpose_body_25 = human::pose::model::openpose_body_25;

        if (poseModelId == openpose_body_25::ModelId)
        {
            return openpose_body_25::JointNames.to_name(openpose_body_25::Joints::WristLeft);
        }

        if (poseModelId == k4a_bt_body_32::ModelId)
        {
            return k4a_bt_body_32::JointNames.to_name(k4a_bt_body_32::Joints::WristLeft);
        }

        ARMARX_ERROR << "Unknown pose model ID '" << poseModelId << "'.";
        return std::nullopt;
    }

    std::optional<std::string>
    rightWristKeypoint(const std::string& poseModelId)
    {
        namespace k4a_bt_body_32 = human::pose::model::k4a_bt_body_32;
        namespace openpose_body_25 = human::pose::model::openpose_body_25;

        if (poseModelId == openpose_body_25::ModelId)
        {
            return openpose_body_25::JointNames.to_name(openpose_body_25::Joints::WristRight);
        }

        if (poseModelId == k4a_bt_body_32::ModelId)
        {
            return k4a_bt_body_32::JointNames.to_name(k4a_bt_body_32::Joints::WristRight);
        }

        ARMARX_ERROR << "Unknown pose model ID '" << poseModelId << "'.";
        return std::nullopt;
    }

    std::optional<std::string>
    pelvisKeypoint(const std::string& poseModelId)
    {
        namespace k4a_bt_body_32 = human::pose::model::k4a_bt_body_32;
        namespace openpose_body_25 = human::pose::model::openpose_body_25;

        if (poseModelId == openpose_body_25::ModelId)
        {
            return openpose_body_25::JointNames.to_name(openpose_body_25::Joints::Pelvis);
        }

        if (poseModelId == k4a_bt_body_32::ModelId)
        {
            return k4a_bt_body_32::JointNames.to_name(k4a_bt_body_32::Joints::Pelvis);
        }

        ARMARX_ERROR << "Unknown pose model ID '" << poseModelId << "'.";
        return std::nullopt;
    }


} // namespace armarx::view_selection::target_provider::handover::util
