/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2022
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <armarx/view_selection/gaze_targets/GazeTarget.h>

#include <VisionX/libraries/armem_human/types.h>

namespace armarx::view_selection::targets
{

    /**
     * @brief Gaze target based on an engagement-driven gaze model based on Yumak et al.,2017
     * 
     * See:
     * 
     *   Yumak, Zerrin, Bram van den Brink, and Arjan Egges. "Autonomous social gaze model for an interactive virtual character in real‐life settings." 
     *   Computer Animation and Virtual Worlds (2017)
     * 
     */
    class Human
    {
    public:
        struct Params
        {
            double distanceMin = 500; // [mm]
            double distanceMax = 4'500; // [mm]
        };

        Human(const armem::human::HumanPose& human,
              const Eigen::Vector3f& robotPosition,
              const Params& params);

        std::optional<gaze_targets::GazeTarget> asGazeTarget() const;

    private:
        double priority(const Eigen::Vector3f& globalHeadPosition) const;

        const armem::human::HumanPose human_;
        const Eigen::Vector3f robotPosition_;

        const Params params_;
    };
} // namespace armarx::view_selection::targets
